#!/usr/bin/env python

__revision__ = '1'

""" INIClass Preview 1

<description soon>
<documentation soon>
"""

from ConfigParser import SafeConfigParser

class INIClass:
    def __init__(self):
        # Initialize the parser
        self.parser = SafeConfigParser()
        self.sections = {}
        self['ROOT'] = {}
        
    def __setitem__(self, item, value):
        # Add section to parser
        self.parser.add_section(item)
        
        # Create new Section class
        self.sections[item] = Section(item, self.parser, value)
    
    def __delitem__(self, item):
        # Does the key exist?
        if self.sections.has_key(item):
            # Remove section
            self.parser.remove_section(item)
        else:
            # Raise NameError
            raise NameError, 'Section %s doesn\'t exist' % item
    
    def __getitem__(self, item):
        # Does the key exist?
        if self.sections.has_key(item):
            return self.sections[item]
        else:
            raise NameError, 'Section %s doesn\'t exist' % item
    
    def __processDict(self, dictionary, section = False):
        for k in dictionary:
            v = dictionary[k]
            # Is the key a dictionary?
            if type(v) == type({}) and not section:
                # Does the section already exist?
                if not self.parser.has_section(k):
                    self[k] = {}
                
                # Process dictionary
                self.__processDict(v, k)
            else:
                # Add the option to the section
                self.parser.set(section, k, v)
    
    def flush(self):
        # Initialize the parser + sections
        self.parser = SafeConfigParser()
        self.sections = {}
        self['ROOT'] = {}
        
        # Remove the file object
        del self.fileobj
        
    
    def importFile(self, filename):
        # Read the file into the parser
        self.parser.read(filename)
    
    def importDictionary(self, dictionary):
        if type(dictionary) != type({}):
            # Isn't a dictionary, raise an error
            raise TypeError, 'Cannot import a non-dictionary.'
        else:
            # Process the dictionary
            self.__processDict(dictionary, False)
            
    def saveToFile(self, filename):
        # Create file object + write to file
        self.fileobj = open(filename, 'w')
        self.parser.write(self.fileobj)
    
    def getSections(self):        
        # Get options
        for k in self.parser.sections():
            # Does key already exist?
            if self.sections.has_key(k):
                continue
            
            self.sections[k] = Section(k, self.parser, {})
            
        # Return dict
        return self.sections
    
    def get(self):
        return self.getSections()

class Section:
    def __init__(self, section, parser, value):
        # Initialize needed vars
        self.section = section
        self.parser = parser
        
        # Add options
        for k in value:
            v = value[k]
            self.parser.set(section, k, v)
        
    def __getitem__(self, option):
        if self.parser.has_option(self.section, option):
            return self.parser.get(self.section, option)
        else:
            raise NameError, 'Option %s doesn\'t exist' % option
        
    def __setitem__(self, item, value):
        # Add section to parser
        self.parser.set(self.section, item, value)
        
    def __delitem__(self, item):
        # Does the key exist?
        if self.parser.has_option(self.section, item):
            # Remove section
            self.parser.remove_option(self.section, item)
        else:
            # Raise NameError
            raise NameError, 'Option %s doesn\'t exist' % item
        
    def __dict__(self):
        return self.getOptions()
        
    def importDictionary(self, value):
        # Add options
        for k in value:
            v = value[k]
            
            self.parser.set(self.section, k, v)
            
    def getOptions(self):
        # Return dict
        returnDict = {}
        
        # Get options
        for k in self.parser.options(self.section):
            returnDict[k] = self.parser.get(self.section, k)
            
        # Return dict
        return returnDict
    
    def get(self):
        return self.getOptions()